from typing import Dict, Optional
from .htcpcp_error import HtpcpError
from .constants import PROTOCOL, COFFEE_SCHEMES, ALLOWED_METHODS


class Request:
    def __init__(
        self,
        method: Optional[str] = None,
        url: Optional[str] = None,
        protocol: str = PROTOCOL,
        headers: Optional[Dict[str, str]] = None,
        payload: Optional[str] = None,
        raw_request: Optional[str] = None
    ) -> None:
        self.method = method or ""
        self.url = url or ""
        self.protocol = protocol
        self.headers: Dict[str, str] = headers or {}
        self.payload = payload

        if raw_request:  # If a raw request is provided, parse it.
            self._parse_request(raw_request)
        # Ensure required fields for new requests.
        elif not (self.method and self.url):
            raise ValueError(
                "Either 'raw_request' or 'method' and 'url' must be provided.")

    def _parse_request(self, raw_request: str) -> None:
        """Parse a raw request string into components."""
        lines = raw_request.split("\n")

        if len(lines) < 1:
            raise HtpcpError("Invalid request: Request is empty.", 400)

        request_line = lines[0].strip()
        self._parse_request_line(request_line)

        header_and_body = raw_request.split("\n\n", 1)
        self._parse_headers(header_and_body[0])

        if len(header_and_body) == 2:
            self.payload = header_and_body[1].strip()

    def _parse_request_line(self, request_line: str) -> None:
        """Parse the request line (e.g., 'GET /coffee HTCPCP/1.0')."""
        parts = request_line.split(" ")

        if len(parts) != 3:
            raise HtpcpError("Invalid request line format.", 400)

        self.method, self.url, self.protocol = parts

        if self.protocol != PROTOCOL:
            raise HtpcpError(f"Invalid protocol: {self.protocol}", 400)

    def _parse_headers(self, header_block: str) -> None:
        """Parse headers from the header block."""
        header_lines = header_block.split("\n")[1:]

        for line in header_lines:
            if ":" in line:
                key, value = map(str.strip, line.split(":", 1))
                self.headers[key.lower()] = value

    def validate(self) -> None:
        """Validate the parsed or constructed request."""
        if not any(self.url.startswith(f"{scheme}://") for scheme in COFFEE_SCHEMES):
            raise HtpcpError("Invalid URL scheme.", 400)

        if self.method not in ALLOWED_METHODS:
            raise HtpcpError(f"Unsupported method: {self.method}", 400)

    def __str__(self) -> str:
        """Convert the request to its raw string representation."""
        request_line = f"{self.method} {self.url} {self.protocol}"
        headers = "\n".join(f"{key}: {value}" for key,
                            value in self.headers.items())
        body = f"\n\n{self.payload}" if self.payload else ""
        return f"{request_line}\n{headers}{body}"
