from typing import Dict, Optional
from .constants import RESPONSES, PROTOCOL


class Response:
    def __init__(
        self,
        status: Optional[int] = None,
        body: Optional[str] = None,
        headers: Optional[Dict[str, str]] = None,
        raw_response: Optional[str] = None
    ) -> None:
        self.status = status or 200
        self.body = body or ""
        self.headers: Dict[str, str] = headers or {}

        if raw_response:  # Parse the raw response string if provided.
            self._parse_response(raw_response)
        elif self.status not in RESPONSES:
            raise ValueError(f"Invalid status code: {self.status}")

    def _parse_response(self, raw_response: str) -> None:
        """Parse a raw HTTP/HTCPCP response string."""
        lines = raw_response.split("\r\n")

        if not lines or len(lines) < 1:
            raise ValueError(
                "Invalid response: Response is empty or malformed.")

        # Parse status line
        status_line = lines[0]
        self._parse_status_line(status_line)

        # Separate headers and body
        header_and_body = raw_response.split("\r\n\r\n", 1)
        self._parse_headers(header_and_body[0])

        if len(header_and_body) == 2:
            self.body = header_and_body[1].strip()

    def _parse_status_line(self, status_line: str) -> None:
        """Parse the status line (e.g., 'HTCPCP/1.0 200 OK')."""
        parts = status_line.split(" ", 2)

        if len(parts) < 3 or parts[0] != PROTOCOL:
            raise ValueError(f"Invalid status line: {status_line}")

        try:
            self.status = int(parts[1])
        except ValueError as e:
            print(e)
            raise e

        if self.status not in RESPONSES:
            raise ValueError(f"Unknown status code: {self.status}")

    def _parse_headers(self, header_block: str) -> None:
        """Parse headers from the header block."""
        header_lines = header_block.split("\r\n")[1:]

        for line in header_lines:
            if ":" in line:
                key, value = map(str.strip, line.split(":", 1))
                self.headers[key.lower()] = value

    def validate(self) -> None:
        """Validate the constructed or parsed response."""
        if self.status not in RESPONSES:
            raise ValueError(f"Invalid status code: {self.status}")

        if not self.headers:
            raise ValueError("Response must have at least one header.")

        if "content-length" in self.headers and not self.body:
            raise ValueError(
                "Content-Length header specified but no body provided.")

    def __str__(self) -> str:
        """Convert the response to its raw string representation."""
        status_line = f"{PROTOCOL} {self.status} {RESPONSES[self.status]}"
        headers = "\r\n".join(f"{key}: {value}" for key,
                              value in self.headers.items())
        body = f"\r\n\r\n{self.body}" if self.body else ""
        return f"{status_line}\r\n{headers}{body}"
