
# TODO 1. Finalize the `Request` Class
#    - Parse the request-line to extract `method`, `url`, and `protocol`.
#    - Validate the protocol is `HTCPCP/1.0`.
#    - Parse headers into a dictionary and validate required ones.
#    - Handle payload extraction based on `Content-Length` header.
#
# TODO 2. Validate Requests
#    - Ensure URL uses a valid scheme (`coffee://`, `caf%C3%E8://`, etc.).
#    - Check method validity against `ALLOWED_METHODS`.
#    - Validate the path and extract query parameters like `additions`.
#
# TODO 3. Handle URL Parameters
#    - Parse query parameters from the URL.
#    - Validate `additions` against a list of allowed values (e.g., cream, sugar).
#    - Respond with 400 Bad Request for invalid additions.
#
# TODO 4. Implement Proper HTCPCP Responses
#    - Support dynamic 200 OK responses for valid requests.
#    - Add error handling for 400, 405, 418, and 500 responses.
#
# TODO 5. Expand `RESPONSES` Dictionary
#    - Add entries dynamically for specific paths and parameters (e.g., `/coffee?additions=cream`).
#
# TODO 6. Create a Simple Client
#    - Use Python's `socket` library to send manual HTCPCP requests.
#    - Include options to customize headers, methods, and payloads.
#
# TODO 7. Add Compliance Checks
#    - Verify mandatory headers like `Accept` and `Content-Type`.
#    - Handle `Content-Length` correctly.
#    - Validate supported `Content-Type` values.
#
# TODO 8. Improve `parse_request` Function
#    - Refactor it to call `Request` methods for parsing and validation.
#
# TODO 9. Support POST Requests
#    - Parse and validate the payload (e.g., JSON format).
#    - Add logic to return appropriate responses for POST requests.
#
# TODO 10. Testing & Error Handling
#    - Write `pytest` cases to automate server testing.
#    - Ensure all exceptions are handled gracefully.
#
# TODO 11. Add Logging (Bonus)
#    - Log incoming requests, responses, and errors for debugging.
import re
import socket

from utils.http_date import get_http_date
from ..common.htcpcp_error import HtpcpError

from ..common.constants import PROTOCOL, RESPONSES
from ..common.request import Request
from ..common.response import Response

# Constants


NAME = "CoffeeMaker"
HOST = '127.0.0.1'
PORT = 6969



ERRORS = {
    "SCHEME_MISMATCH": "HTTP/1.1 400 Bad Request\r\nContent-Type: text/plain\r\n\r\nInvalid scheme. Use coffee://.",
    "NOT_FOUND": "HTTP/1.1 404 Not Found\r\nContent-Type: text/plain\r\n\r\nCommand not recognized."
}


class Server():
    def __init__(self, host: str, port: int):
        self.host = host
        self.port = port
        self.queue = []

    def parse_request(self, raw_request: str) -> Response:
        """
        Parse the raw request and validate the COFFEE scheme.
        Return a proper response or error.
        """
        response = Response()

        try:
            request = Request(raw_request)
        except HtpcpError as e:
            print(e)
            response.status = e.status


        return response

    def start_server(self):
        """Start the TCP server to handle COFFEE protocol requests."""
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as socket_server:
            socket_server.bind((self.host, self.port))
            socket_server.listen()
            print(f"Server started at {HOST}:{PORT}")

            while True:
                conn, addr = socket_server.accept()
                with conn:
                    print(f"Connected by {addr}")
                    request = conn.recv(1024).decode("utf-8")
                    print(f"Request:\n{request}")

                    # Process the request and send the response
                    response = self.parse_request(request)
                    conn.sendall(str(response).encode("utf-8"))


if __name__ == "__main__":
    server = Server(HOST, PORT)
    server.start_server()
